@@ -6,12 +6,13 @@
 Simple Mail Transfer Protocol implementation.
 """
 
-import time, re, base64, types, socket, os, random, rfc822
+import time, re, base64, types, socket, os, random
 import binascii
+import email.utils
 import warnings
 from email.base64MIME import encode as encode_base64
 
-from zope.interface import implements, Interface
+from zope.interface import implementer
 
 from twisted.copyright import longversion
 from twisted.protocols import basic
@@ -23,6 +24,18 @@ from twisted.internet import reactor
 from twisted.internet.interfaces import ITLSTransport, ISSLTransport
 from twisted.python import log
 from twisted.python import util
+from twisted.mail.interfaces import (IClientAuthentication,
+                                     IMessageSMTP as IMessage,
+                                     IMessageDeliveryFactory, IMessageDelivery)
+from twisted.mail._cred import (CramMD5ClientAuthenticator, LOGINAuthenticator,
+                                LOGINCredentials as _lcredentials)
+from twisted.mail._except import (
+    AUTHDeclinedError, AUTHRequiredError, AddressError,
+    AuthenticationError, EHLORequiredError, ESMTPClientError,
+    SMTPAddressError, SMTPBadRcpt, SMTPBadSender, SMTPClientError,
+    SMTPConnectError, SMTPDeliveryError, SMTPError, SMTPServerError,
+    SMTPTimeoutError, SMTPTLSError as TLSError, TLSRequiredError,
+    SMTPProtocolError)
 
 from twisted import cred
 from twisted.python.runtime import platform
@@ -32,6 +45,29 @@ try:
 except ImportError:
     from StringIO import StringIO
 
+
+__all__ = [
+    'AUTHDeclinedError', 'AUTHRequiredError', 'AddressError',
+    'AuthenticationError',  'EHLORequiredError', 'ESMTPClientError',
+    'SMTPAddressError', 'SMTPBadRcpt', 'SMTPBadSender', 'SMTPClientError',
+    'SMTPConnectError', 'SMTPDeliveryError', 'SMTPError', 'SMTPServerError',
+    'SMTPTimeoutError', 'TLSError', 'TLSRequiredError', 'SMTPProtocolError',
+
+    'IClientAuthentication', 'IMessage', 'IMessageDelivery',
+    'IMessageDeliveryFactory',
+
+    'CramMD5ClientAuthenticator', 'LOGINAuthenticator', 'LOGINCredentials',
+    'PLAINAuthenticator',
+
+    'Address', 'User', 'sendmail', 'SenderMixin',
+    'ESMTP', 'ESMTPClient', 'ESMTPSender', 'ESMTPSenderFactory',
+    'SMTP', 'SMTPClient', 'SMTPFactory',  'SMTPSender', 'SMTPSenderFactory',
+
+    'idGenerator', 'messageid', 'quoteaddr', 'rfc822date', 'xtextStreamReader',
+    'xtextStreamWriter', 'xtext_codec', 'xtext_decode', 'xtext_encode'
+]
+
+
 # Cache the hostname (XXX Yes - this is broken)
 if platform.isMacOSX():
     # On OS X, getfqdn() is ridiculously slow - use the
@@ -43,226 +79,6 @@ else:
 # Used for fast success code lookup
 SUCCESS = dict.fromkeys(xrange(200,300))
 
-class IMessageDelivery(Interface):
-    def receivedHeader(helo, origin, recipients):
-        """
-        Generate the Received header for a message
-
-        @type helo: C{(str, str)}
-        @param helo: The argument to the HELO command and the client's IP
-        address.
-
-        @type origin: C{Address}
-        @param origin: The address the message is from
-
-        @type recipients: C{list} of L{User}
-        @param recipients: A list of the addresses for which this message
-        is bound.
-
-        @rtype: C{str}
-        @return: The full \"Received\" header string.
-        """
-
-    def validateTo(user):
-        """
-        Validate the address for which the message is destined.
-
-        @type user: C{User}
-        @param user: The address to validate.
-
-        @rtype: no-argument callable
-        @return: A C{Deferred} which becomes, or a callable which
-        takes no arguments and returns an object implementing C{IMessage}.
-        This will be called and the returned object used to deliver the
-        message when it arrives.
-
-        @raise SMTPBadRcpt: Raised if messages to the address are
-        not to be accepted.
-        """
-
-    def validateFrom(helo, origin):
-        """
-        Validate the address from which the message originates.
-
-        @type helo: C{(str, str)}
-        @param helo: The argument to the HELO command and the client's IP
-        address.
-
-        @type origin: C{Address}
-        @param origin: The address the message is from
-
-        @rtype: C{Deferred} or C{Address}
-        @return: C{origin} or a C{Deferred} whose callback will be
-        passed C{origin}.
-
-        @raise SMTPBadSender: Raised of messages from this address are
-        not to be accepted.
-        """
-
-class IMessageDeliveryFactory(Interface):
-    """An alternate interface to implement for handling message delivery.
-
-    It is useful to implement this interface instead of L{IMessageDelivery}
-    directly because it allows the implementor to distinguish between
-    different messages delivery over the same connection.  This can be
-    used to optimize delivery of a single message to multiple recipients,
-    something which cannot be done by L{IMessageDelivery} implementors
-    due to their lack of information.
-    """
-    def getMessageDelivery():
-        """Return an L{IMessageDelivery} object.
-
-        This will be called once per message.
-        """
-
-class SMTPError(Exception):
-    pass
-
-
-
-class SMTPClientError(SMTPError):
-    """Base class for SMTP client errors.
-    """
-    def __init__(self, code, resp, log=None, addresses=None, isFatal=False, retry=False):
-        """
-        @param code: The SMTP response code associated with this error.
-        @param resp: The string response associated with this error.
-
-        @param log: A string log of the exchange leading up to and including
-            the error.
-        @type log: L{str}
-
-        @param isFatal: A boolean indicating whether this connection can
-            proceed or not.  If True, the connection will be dropped.
-
-        @param retry: A boolean indicating whether the delivery should be
-            retried.  If True and the factory indicates further retries are
-            desirable, they will be attempted, otherwise the delivery will
-            be failed.
-        """
-        self.code = code
-        self.resp = resp
-        self.log = log
-        self.addresses = addresses
-        self.isFatal = isFatal
-        self.retry = retry
-
-
-    def __str__(self):
-        if self.code > 0:
-            res = ["%.3d %s" % (self.code, self.resp)]
-        else:
-            res = [self.resp]
-        if self.log:
-            res.append(self.log)
-            res.append('')
-        return '\n'.join(res)
-
-
-class ESMTPClientError(SMTPClientError):
-    """Base class for ESMTP client errors.
-    """
-
-class EHLORequiredError(ESMTPClientError):
-    """The server does not support EHLO.
-
-    This is considered a non-fatal error (the connection will not be
-    dropped).
-    """
-
-class AUTHRequiredError(ESMTPClientError):
-    """Authentication was required but the server does not support it.
-
-    This is considered a non-fatal error (the connection will not be
-    dropped).
-    """
-
-class TLSRequiredError(ESMTPClientError):
-    """Transport security was required but the server does not support it.
-
-    This is considered a non-fatal error (the connection will not be
-    dropped).
-    """
-
-class AUTHDeclinedError(ESMTPClientError):
-    """The server rejected our credentials.
-
-    Either the username, password, or challenge response
-    given to the server was rejected.
-
-    This is considered a non-fatal error (the connection will not be
-    dropped).
-    """
-
-class AuthenticationError(ESMTPClientError):
-    """An error occurred while authenticating.
-
-    Either the server rejected our request for authentication or the
-    challenge received was malformed.
-
-    This is considered a non-fatal error (the connection will not be
-    dropped).
-    """
-
-class TLSError(ESMTPClientError):
-    """An error occurred while negiotiating for transport security.
-
-    This is considered a non-fatal error (the connection will not be
-    dropped).
-    """
-
-class SMTPConnectError(SMTPClientError):
-    """Failed to connect to the mail exchange host.
-
-    This is considered a fatal error.  A retry will be made.
-    """
-    def __init__(self, code, resp, log=None, addresses=None, isFatal=True, retry=True):
-        SMTPClientError.__init__(self, code, resp, log, addresses, isFatal, retry)
-
-class SMTPTimeoutError(SMTPClientError):
-    """Failed to receive a response from the server in the expected time period.
-
-    This is considered a fatal error.  A retry will be made.
-    """
-    def __init__(self, code, resp, log=None, addresses=None, isFatal=True, retry=True):
-        SMTPClientError.__init__(self, code, resp, log, addresses, isFatal, retry)
-
-class SMTPProtocolError(SMTPClientError):
-    """The server sent a mangled response.
-
-    This is considered a fatal error.  A retry will not be made.
-    """
-    def __init__(self, code, resp, log=None, addresses=None, isFatal=True, retry=False):
-        SMTPClientError.__init__(self, code, resp, log, addresses, isFatal, retry)
-
-class SMTPDeliveryError(SMTPClientError):
-    """Indicates that a delivery attempt has had an error.
-    """
-
-class SMTPServerError(SMTPError):
-    def __init__(self, code, resp):
-        self.code = code
-        self.resp = resp
-
-    def __str__(self):
-        return "%.3d %s" % (self.code, self.resp)
-
-class SMTPAddressError(SMTPServerError):
-    def __init__(self, addr, code, resp):
-        SMTPServerError.__init__(self, code, resp)
-        self.addr = Address(addr)
-
-    def __str__(self):
-        return "%.3d <%s>... %s" % (self.code, self.addr, self.resp)
-
-class SMTPBadRcpt(SMTPAddressError):
-    def __init__(self, addr, code=550,
-                 resp='Cannot receive for specified address'):
-        SMTPAddressError.__init__(self, addr, code, resp)
-
-class SMTPBadSender(SMTPAddressError):
-    def __init__(self, addr, code=550, resp='Sender not acceptable'):
-        SMTPAddressError.__init__(self, addr, code, resp)
 
 def rfc822date(timeinfo=None,local=1):
     """
@@ -303,14 +119,19 @@ def rfc822date(timeinfo=None,local=1):
         timeinfo[0], timeinfo[3], timeinfo[4], timeinfo[5],
         tzhr, tzmin)
 
+
+
 def idGenerator():
     i = 0
     while True:
         yield i
         i += 1
 
+
+
 def messageid(uniq=None, N=idGenerator().next):
-    """Return a globally unique random string in RFC 2822 Message-ID format
+    """
+    Return a globally unique random string in RFC 2822 Message-ID format
 
     <datetime.pid.random@host.dom.ain>
 
@@ -326,15 +147,18 @@ def messageid(uniq=None, N=idGenerator().next):
 
     return '<%s.%s.%s%s.%s@%s>' % (datetime, pid, rand, uniq, N(), DNSNAME)
 
+
+
 def quoteaddr(addr):
-    """Turn an email address, possibly with realname part etc, into
+    """
+    Turn an email address, possibly with realname part etc, into
     a form suitable for and SMTP envelope.
     """
 
     if isinstance(addr, Address):
         return '<%s>' % str(addr)
 
-    res = rfc822.parseaddr(addr)
+    res = email.utils.parseaddr(addr)
 
     if res == (None, None):
         # It didn't parse, use it as-is
@@ -344,8 +168,6 @@ def quoteaddr(addr):
 
 COMMAND, DATA, AUTH = 'COMMAND', 'DATA', 'AUTH'
 
-class AddressError(SMTPError):
-    "Parse error in address"
 
 # Character classes for parsing addresses
 atom = r"[-A-Za-z0-9!\#$%&'*+/=?^_`{|}~]"
@@ -370,6 +192,7 @@ class Address:
                           )+|.) # or any single character''',re.X)
     atomre = re.compile(atom) # match any one atom character
 
+
     def __init__(self, addr, defaultDomain=None):
         if isinstance(addr, User):
             addr = addr.dest
@@ -424,8 +247,11 @@ class Address:
 
     dequotebs = re.compile(r'\\(.)')
 
+
     def dequote(self,addr):
-        """Remove RFC-2821 quotes from address."""
+        """
+        Remove RFC-2821 quotes from address.
+        """
         res = []
 
         atl = filter(None,self.tstring.split(str(addr)))
@@ -440,21 +266,25 @@ class Address:
 
         return ''.join(res)
 
+
     def __str__(self):
         if self.local or self.domain:
             return '@'.join((self.local, self.domain))
         else:
             return ''
 
+
     def __repr__(self):
         return "%s.%s(%s)" % (self.__module__, self.__class__.__name__,
                               repr(str(self)))
 
+
+
 class User:
-    """Hold information about and SMTP message recipient,
+    """
+    Hold information about and SMTP message recipient,
     including information on where the message came from
     """
-
     def __init__(self, destination, helo, protocol, orig):
         host = getattr(protocol, 'host', None)
         self.dest = Address(destination, host)
@@ -465,8 +295,10 @@ class User:
         else:
             self.orig = Address(orig, host)
 
+
     def __getstate__(self):
-        """Helper for pickle.
+        """
+        Helper for pickle.
 
         protocol isn't picklabe, but we want User to be, so skip it in
         the pickle.
@@ -479,24 +311,6 @@ class User:
     def __str__(self):
         return str(self.dest)
 
-class IMessage(Interface):
-    """Interface definition for messages that can be sent via SMTP."""
-
-    def lineReceived(line):
-        """handle another line"""
-
-    def eomReceived():
-        """handle end of message
-
-        return a deferred. The deferred should be called with either:
-        callback(string) or errback(error)
-        """
-
-    def connectionLost():
-        """handle message truncated
-
-        semantics should be to discard the message
-        """
 
 class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
     """
@@ -535,14 +349,17 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         self.delivery = delivery
         self.deliveryFactory = deliveryFactory
 
+
     def timeoutConnection(self):
         msg = '%s Timeout. Try talking faster next time!' % (self.host,)
         self.sendCode(421, msg)
         self.transport.loseConnection()
 
+
     def greeting(self):
         return '%s NO UCE NO UBE NO RELAY PROBES' % (self.host,)
 
+
     def connectionMade(self):
         # Ensure user-code always gets something sane for _helo
         peer = self.transport.getPeer()
@@ -554,8 +371,11 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         self.sendCode(220, self.greeting())
         self.setTimeout(self.timeout)
 
+
     def sendCode(self, code, message=''):
-        "Send an SMTP code with a message."
+        """
+        Send an SMTP code with a message.
+        """
         lines = message.splitlines()
         lastline = lines[-1:]
         for line in lines[:-1]:
@@ -563,10 +383,12 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         self.sendLine('%3.3d %s' % (code,
                                     lastline and lastline[0] or ''))
 
+
     def lineReceived(self, line):
         self.resetTimeout()
         return getattr(self, 'state_' + self.mode)(line)
 
+
     def state_COMMAND(self, line):
         # Ignore leading and trailing whitespace, as well as an arbitrary
         # amount of whitespace between the command and its argument, though
@@ -583,12 +405,15 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         else:
             self.sendSyntaxError()
 
+
     def sendSyntaxError(self):
         self.sendCode(500, 'Error: bad syntax')
 
+
     def lookupMethod(self, command):
         return getattr(self, 'do_' + command.upper(), None)
 
+
     def lineLengthExceeded(self, line):
         if self.mode is DATA:
             for message in self.__messages:
@@ -597,9 +422,11 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             del self.__messages
         self.sendCode(500, 'Line too long')
 
+
     def do_UNKNOWN(self, rest):
         self.sendCode(500, 'Command not implemented')
 
+
     def do_HELO(self, rest):
         peer = self.transport.getPeer()
         try:
@@ -611,6 +438,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         self._to = []
         self.sendCode(250, '%s Hello %s, nice to meet you' % (self.host, host))
 
+
     def do_QUIT(self, rest):
         self.sendCode(221, 'See you later')
         self.transport.loseConnection()
@@ -691,12 +519,14 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             callbackArgs=(user,)
         )
 
+
     def _cbToValidate(self, to, user=None, code=250, msg='Recipient address accepted'):
         if user is None:
             user = to
         self._to.append((user, to))
         self.sendCode(code, msg)
 
+
     def _ebToValidate(self, failure):
         if failure.check(SMTPBadRcpt, SMTPServerError):
             self.sendCode(failure.value.code, failure.value.resp)
@@ -707,6 +537,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
                 'Requested action aborted: local error in processing'
             )
 
+
     def _disconnect(self, msgs):
         for msg in msgs:
             try:
@@ -715,6 +546,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
                 log.msg("msg raised exception from connectionLost")
                 log.err()
 
+
     def do_DATA(self, rest):
         if self._from is None or (not self._to):
             self.sendCode(503, 'Must have valid receiver and originator')
@@ -755,6 +587,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             fmt = 'Receiving message for delivery: from=%s to=%s'
             log.msg(fmt % (origin, [str(u) for (u, f) in recipients]))
 
+
     def connectionLost(self, reason):
         # self.sendCode(421, 'Dropping connection.') # This does nothing...
         # Ideally, if we (rather than the other side) lose the connection,
@@ -775,11 +608,13 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             self._onLogout = None
         self.setTimeout(None)
 
+
     def do_RSET(self, rest):
         self._from = None
         self._to = []
         self.sendCode(250, 'I remember nothing.')
 
+
     def dataLineReceived(self, line):
         if line[:1] == '.':
             if line == '.':
@@ -825,6 +660,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
                 message.connectionLost()
     state_DATA = dataLineReceived
 
+
     def _messageHandled(self, resultList):
         failures = 0
         for (success, result) in resultList:
@@ -944,6 +780,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             return self.delivery.validateTo(user)
         raise SMTPBadRcpt(user)
 
+
     def receivedHeader(self, helo, origin, recipients):
         if self.delivery is not None:
             return self.delivery.receivedHeader(helo, origin, recipients)
@@ -963,7 +800,9 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
 
 
 class SMTPFactory(protocol.ServerFactory):
-    """Factory for SMTP."""
+    """
+    Factory for SMTP.
+    """
 
     # override in instances or subclasses
     domain = DNSNAME
@@ -975,12 +814,15 @@ class SMTPFactory(protocol.ServerFactory):
     def __init__(self, portal = None):
         self.portal = portal
 
+
     def buildProtocol(self, addr):
         p = protocol.ServerFactory.buildProtocol(self, addr)
         p.portal = self.portal
         p.host = self.domain
         return p
 
+
+
 class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
     """
     SMTP client for sending emails.
@@ -988,7 +830,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
     After the client has connected to the SMTP server, it repeatedly calls
     L{SMTPClient.getMailFrom}, L{SMTPClient.getMailTo} and
     L{SMTPClient.getMailData} and uses this information to send an email.
-    It then calls L{SMTPClient.getMailFrom} again; if it returns C{None}, the
+    It then calls L{SMTPClient.getMailFrom} again; if it returns L{None}, the
     client will disconnect, otherwise it will continue as normal i.e. call
     L{SMTPClient.getMailTo} and L{SMTPClient.getMailData} and send a new email.
     """
@@ -1009,6 +851,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         self.code = -1
         self.log = util.LineLog(logsize)
 
+
     def sendLine(self, line):
         # Log sendLine only if you are in debug mode for performance
         if self.debug:
@@ -1016,6 +859,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
 
         basic.LineReceiver.sendLine(self,line)
 
+
     def connectionMade(self):
         self.setTimeout(self.timeout)
 
@@ -1023,17 +867,22 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         self._okresponse = self.smtpState_helo
         self._failresponse = self.smtpConnectionFailed
 
+
     def connectionLost(self, reason=protocol.connectionDone):
-        """We are no longer connected"""
+        """
+        We are no longer connected
+        """
         self.setTimeout(None)
         self.mailFile = None
 
+
     def timeoutConnection(self):
         self.sendError(
             SMTPTimeoutError(
                 -1, "Timeout waiting for SMTP server response",
                  self.log.str()))
 
+
     def lineReceived(self, line):
         self.resetTimeout()
 
@@ -1069,20 +918,24 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         self.resp = []
         return why
 
+
     def smtpConnectionFailed(self, code, resp):
         self.sendError(SMTPConnectError(code, resp, self.log.str()))
 
+
     def smtpTransferFailed(self, code, resp):
         if code < 0:
             self.sendError(SMTPProtocolError(code, resp, self.log.str()))
         else:
             self.smtpState_msgSent(code, resp)
 
+
     def smtpState_helo(self, code, resp):
         self.sendLine('HELO ' + self.identity)
         self._expected = SUCCESS
         self._okresponse = self.smtpState_from
 
+
     def smtpState_from(self, code, resp):
         self._from = self.getMailFrom()
         self._failresponse = self.smtpTransferFailed
@@ -1094,9 +947,11 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
             # All messages have been sent, disconnect
             self._disconnectFromServer()
 
+
     def smtpState_disconnect(self, code, resp):
         self.transport.loseConnection()
 
+
     def smtpState_to(self, code, resp):
         self.toAddresses = iter(self.getMailTo())
         self.toAddressesResult = []
@@ -1106,6 +961,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         self.lastAddress = None
         return self.smtpState_toOrData(0, '')
 
+
     def smtpState_toOrData(self, code, resp):
         if self.lastAddress is not None:
             self.toAddressesResult.append((self.lastAddress, code, resp))
@@ -1123,6 +979,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         else:
             self.sendLine('RCPT TO:%s' % quoteaddr(self.lastAddress))
 
+
     def smtpState_data(self, code, resp):
         s = basic.FileSender()
         d = s.beginFileTransfer(
@@ -1145,6 +1002,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         self._expected = SUCCESS
         self._okresponse = self.smtpState_from
 
+
     ##
     ## Helpers for FileSender
     ##
@@ -1159,6 +1017,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         self.resetTimeout()
         return chunk.replace('\n', '\r\n').replace('\r\n.', '\r\n..')
 
+
     def finishedFileTransfer(self, lastsent):
         if lastsent != '\n':
             line = '\r\n.'
@@ -1166,23 +1025,32 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
             line = '.'
         self.sendLine(line)
 
+
     ##
     # these methods should be overridden in subclasses
     def getMailFrom(self):
-        """Return the email address the mail is from."""
+        """
+        Return the email address the mail is from.
+        """
         raise NotImplementedError
 
+
     def getMailTo(self):
-        """Return a list of emails to send to."""
+        """
+        Return a list of emails to send to.
+        """
         raise NotImplementedError
 
+
     def getMailData(self):
-        """Return file-like object containing data of message to be sent.
+        """
+        Return file-like object containing data of message to be sent.
 
         Lines in the file should be delimited by '\\n'.
         """
         raise NotImplementedError
 
+
     def sendError(self, exc):
         """
         If an error occurs before a mail message is sent sendError will be
@@ -1201,7 +1069,8 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
 
 
     def sentMail(self, code, resp, numOk, addresses, log):
-        """Called when an attempt to send an email is completed.
+        """
+        Called when an attempt to send an email is completed.
 
         If some addresses were accepted, code and resp are the response
         to the DATA command. If no addresses were accepted, code is -1
@@ -1216,6 +1085,7 @@ class SMTPClient(basic.LineReceiver, policies.TimeoutMixin):
         """
         raise NotImplementedError
 
+
     def _disconnectFromServer(self):
         self._expected = xrange(0, 1000)
         self._okresponse = self.smtpState_disconnect
@@ -1415,7 +1285,7 @@ class ESMTPClient(SMTPClient):
         @param auth: The Authentication mechanism to register
         @type auth: L{IClientAuthentication} implementor
 
-        @return C{None}
+        @return: L{None}
         """
         self.authenticators.append(auth)
 
@@ -1445,7 +1315,7 @@ class ESMTPClient(SMTPClient):
             server message.
         @type resp: L{bytes}
 
-        @return: C{None}
+        @return: L{None}
         """
         self._expected = SUCCESS
 
@@ -1503,7 +1373,7 @@ class ESMTPClient(SMTPClient):
             are extension identifiers and values are the associated values.
         @type items: L{dict} mapping L{bytes} to L{bytes}
 
-        @return: C{None}
+        @return: L{None}
         """
 
         # has tls        can tls         must tls       result
@@ -1637,7 +1507,6 @@ class ESMTPClient(SMTPClient):
 
 
 class ESMTP(SMTP):
-
     ctx = None
     canStartTLS = False
     startedTLS = False
@@ -1652,6 +1521,7 @@ class ESMTP(SMTP):
         self.authenticated = False
         self.ctx = contextFactory
 
+
     def connectionMade(self):
         SMTP.connectionMade(self)
         self.canStartTLS = ITLSTransport.providedBy(self.transport)
@@ -1668,12 +1538,14 @@ class ESMTP(SMTP):
             ext['STARTTLS'] = None
         return ext
 
+
     def lookupMethod(self, command):
         m = SMTP.lookupMethod(self, command)
         if m is None:
             m = getattr(self, 'ext_' + command.upper(), None)
         return m
 
+
     def listExtensions(self):
         r = []
         for (c, v) in self.extensions().iteritems():
@@ -1685,6 +1557,7 @@ class ESMTP(SMTP):
                 r.append(c)
         return '\n'.join(r)
 
+
     def do_EHLO(self, rest):
         peer = self.transport.getPeer().host
         self._helo = (rest, peer)
@@ -1698,6 +1571,7 @@ class ESMTP(SMTP):
             )
         )
 
+
     def ext_STARTTLS(self, rest):
         if self.startedTLS:
             self.sendCode(503, 'TLS already negotiated')
@@ -1708,6 +1582,7 @@ class ESMTP(SMTP):
         else:
             self.sendCode(454, 'TLS not available')
 
+
     def ext_AUTH(self, rest):
         if self.authenticated:
             self.sendCode(503, 'Already authenticated')
@@ -1808,7 +1683,8 @@ class ESMTP(SMTP):
 
 
 class SenderMixin:
-    """Utility class for sending emails easily.
+    """
+    Utility class for sending emails easily.
 
     Use with SMTPSenderFactory or ESMTPSenderFactory.
     """
@@ -1821,12 +1697,15 @@ class SenderMixin:
         else:
             return None
 
+
     def getMailTo(self):
         return self.factory.toEmail
 
+
     def getMailData(self):
         return self.factory.file
 
+
     def sendError(self, exc):
         # Call the base class to close the connection with the SMTP server
         SMTPClient.sendError(self, exc)
@@ -1841,6 +1720,7 @@ class SenderMixin:
             self.factory.sendFinished = True
             self.factory.result.errback(exc)
 
+
     def sentMail(self, code, resp, numOk, addresses, log):
         # Do not retry, the SMTP server acknowledged the request
         self.factory.sendFinished = True
@@ -1858,6 +1738,7 @@ class SenderMixin:
             self.factory.result.callback((numOk, addresses))
 
 
+
 class SMTPSender(SenderMixin, SMTPClient):
     """
     SMTP protocol that sends a single email based on information it
@@ -1865,6 +1746,7 @@ class SMTPSender(SenderMixin, SMTPClient):
     """
 
 
+
 class SMTPSenderFactory(protocol.ClientFactory):
     """
     Utility factory for sending emails easily.
@@ -1919,16 +1801,20 @@ class SMTPSenderFactory(protocol.ClientFactory):
         self.retries = -retries
         self.timeout = timeout
 
+
     def _removeDeferred(self, result):
         del self.result
         return result
 
+
     def clientConnectionFailed(self, connector, err):
         self._processConnectionError(connector, err)
 
+
     def clientConnectionLost(self, connector, err):
         self._processConnectionError(connector, err)
 
+
     def _processConnectionError(self, connector, err):
         self.currentProtocol = None
         if (self.retries < 0) and (not self.sendFinished):
@@ -1946,6 +1832,7 @@ class SMTPSenderFactory(protocol.ClientFactory):
                 err.value = SMTPConnectError(-1, "Unable to connect to server.")
             self.result.errback(err.value)
 
+
     def buildProtocol(self, addr):
         p = self.protocol(self.domain, self.nEmails*2+2)
         p.factory = self
@@ -1954,6 +1841,7 @@ class SMTPSenderFactory(protocol.ClientFactory):
         self.result.addBoth(self._removeProtocol)
         return p
 
+
     def _removeProtocol(self, result):
         """
         Remove the protocol created in C{buildProtocol}.
@@ -1969,10 +1857,6 @@ class SMTPSenderFactory(protocol.ClientFactory):
 
 
 
-from twisted.mail.imap4 import IClientAuthentication
-from twisted.mail.imap4 import CramMD5ClientAuthenticator, LOGINAuthenticator
-from twisted.mail.imap4 import LOGINCredentials as _lcredentials
-
 class LOGINCredentials(_lcredentials):
     """
     L{LOGINCredentials} generates challenges for I{LOGIN} authentication.
@@ -1988,15 +1872,16 @@ class LOGINCredentials(_lcredentials):
 
 
 
+@implementer(IClientAuthentication)
 class PLAINAuthenticator:
-    implements(IClientAuthentication)
-
     def __init__(self, user):
         self.user = user
 
+
     def getName(self):
         return "PLAIN"
 
+
     def challengeResponse(self, secret, chal=1):
         if chal == 1:
             return "%s\0%s\0%s" % (self.user, self.user, secret)
@@ -2021,12 +1906,14 @@ class ESMTPSender(SenderMixin, ESMTPClient):
 
         self._registerAuthenticators()
 
+
     def _registerAuthenticators(self):
         # Register Authenticator in order from most secure to least secure
         self.registerAuthenticator(CramMD5ClientAuthenticator(self.username))
         self.registerAuthenticator(LOGINAuthenticator(self.username))
         self.registerAuthenticator(PLAINAuthenticator(self.username))
 
+
     def _getContextFactory(self):
         if self.context is not None:
             return self.context
@@ -2118,8 +2005,8 @@ def sendmail(smtphost, from_addr, to_addrs, msg, senderDomainName=None, port=25,
         File-like objects need to support read() and close(). Lines must be
         delimited by '\\n'. If you pass something that doesn't look like a file,
         we try to convert it to a string (so you should be able to pass an
-        L{email.Message} directly, but doing the conversion with
-        L{email.Generator} manually will give you more control over the process).
+        L{email.message} directly, but doing the conversion with
+        L{email.generator} manually will give you more control over the process).
 
     @param senderDomainName: Name by which to identify. If None, try to pick
         something sane (but this depends on external configuration and may not
@@ -2159,6 +2046,7 @@ def sendmail(smtphost, from_addr, to_addrs, msg, senderDomainName=None, port=25,
         # It's not a file
         msg = StringIO(str(msg))
 
+
     def cancel(d):
         """
         Cancel the L{twisted.mail.smtp.sendmail} call, tell the factory not to
@@ -2202,6 +2090,7 @@ def xtext_encode(s, errors=None):
     return (''.join(r), len(s))
 
 
+
 def xtext_decode(s, errors=None):
     """
     Decode the xtext-encoded string C{s}.
@@ -2220,14 +2109,20 @@ def xtext_decode(s, errors=None):
             i += 1
     return (''.join(r), len(s))
 
+
+
 class xtextStreamReader(codecs.StreamReader):
     def decode(self, s, errors='strict'):
         return xtext_decode(s)
 
+
+
 class xtextStreamWriter(codecs.StreamWriter):
     def decode(self, s, errors='strict'):
         return xtext_encode(s)
 
+
+
 def xtext_codec(name):
     if name == 'xtext':
         return (xtext_encode, xtext_decode, xtextStreamReader, xtextStreamWriter)
