@@ -7,7 +7,7 @@ Test HTTP support.
 
 from __future__ import absolute_import, division
 
-import random, cgi, base64
+import random, cgi, base64, calendar
 
 try:
     from urlparse import urlparse, urlunsplit, clear_cache
@@ -44,6 +44,24 @@ class DateTimeTests(unittest.TestCase):
             self.assertEqual(time, time2)
 
 
+    def testStringToDatetime(self):
+        dateStrings = [
+            b"Sun, 06 Nov 1994 08:49:37 GMT",
+            b"06 Nov 1994 08:49:37 GMT",
+            b"Sunday, 06-Nov-94 08:49:37 GMT",
+            b"06-Nov-94 08:49:37 GMT",
+            b"Sunday, 06-Nov-1994 08:49:37 GMT",
+            b"06-Nov-1994 08:49:37 GMT",
+            b"Sun Nov  6 08:49:37 1994",
+            b"Nov  6 08:49:37 1994",
+        ]
+        dateInt = calendar.timegm((1994, 11, 6, 8, 49, 37, 6, 6, 0))
+        for dateString in dateStrings:
+            self.assertEqual(http.stringToDatetime(dateString), dateInt)
+        self.assertEqual(
+            http.stringToDatetime(b"Thursday, 29-Sep-16 17:15:29 GMT"),
+            calendar.timegm((2016, 9, 29, 17, 15, 29, 3, 273, 0)))
+
 
 class DummyHTTPHandler(http.Request):
 
@@ -767,7 +785,7 @@ class IdentityTransferEncodingTests(TestCase):
 
     def test_unknownContentLength(self):
         """
-        If L{_IdentityTransferDecoder} is constructed with C{None} for the
+        If L{_IdentityTransferDecoder} is constructed with L{None} for the
         content length, it passes all data delivered to it through to the data
         callback.
         """
@@ -1784,7 +1802,7 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
 
     def test_getHeaderNotFound(self):
         """
-        L{http.Request.getHeader} returns C{None} when asked for the value of a
+        L{http.Request.getHeader} returns L{None} when asked for the value of a
         request header which is not present.
         """
         req = http.Request(DummyChannel(), False)
@@ -1881,6 +1899,115 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
             "Python 3 has no separate long integer type.")
 
 
+    def test_setLastModifiedNeverSet(self):
+        """
+        When no previous value was set and no 'if-modified-since' value was
+        requested, L{http.Request.setLastModified} takes a timestamp in seconds
+        since the epoch and sets the request's lastModified attribute.
+        """
+        req = http.Request(DummyChannel(), False)
+
+        req.setLastModified(42)
+
+        self.assertEqual(req.lastModified, 42)
+
+
+    def test_setLastModifiedUpdate(self):
+        """
+        If the supplied timestamp is later than the lastModified attribute's
+        value, L{http.Request.setLastModified} updates the lastModifed
+        attribute.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.setLastModified(0)
+
+        req.setLastModified(1)
+
+        self.assertEqual(req.lastModified, 1)
+
+
+    def test_setLastModifiedIgnore(self):
+        """
+        If the supplied timestamp occurs earlier than the current lastModified
+        attribute, L{http.Request.setLastModified} ignores it.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.setLastModified(1)
+
+        req.setLastModified(0)
+
+        self.assertEqual(req.lastModified, 1)
+
+
+    def test_setLastModifiedCached(self):
+        """
+        If the resource is older than the if-modified-since date in the request
+        header, L{http.Request.setLastModified} returns L{http.CACHED}.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.requestHeaders.setRawHeaders(
+            networkString('if-modified-since'),
+                          [b'02 Jan 1970 00:00:00 GMT']
+            )
+
+        result = req.setLastModified(42)
+
+        self.assertEqual(result, http.CACHED)
+
+
+    def test_setLastModifiedNotCached(self):
+        """
+        If the resource is newer than the if-modified-since date in the request
+        header, L{http.Request.setLastModified} returns None
+        """
+        req = http.Request(DummyChannel(), False)
+        req.requestHeaders.setRawHeaders(
+            networkString('if-modified-since'),
+                          [b'01 Jan 1970 00:00:00 GMT']
+            )
+
+        result = req.setLastModified(1000000)
+
+        self.assertEqual(result, None)
+
+
+    def test_setLastModifiedTwiceNotCached(self):
+        """
+        When L{http.Request.setLastModified} is called multiple times, the
+        highest supplied value is honored. If that value is higher than the
+        if-modified-since date in the request header, the method returns None.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.requestHeaders.setRawHeaders(
+            networkString('if-modified-since'),
+                          [b'01 Jan 1970 00:00:01 GMT']
+            )
+        req.setLastModified(1000000)
+
+        result = req.setLastModified(0)
+
+        self.assertEqual(result, None)
+
+
+    def test_setLastModifiedTwiceCached(self):
+        """
+        When L{http.Request.setLastModified} is called multiple times, the
+        highest supplied value is honored. If that value is lower than the
+        if-modified-since date in the request header, the method returns
+        L{http.CACHED}.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.requestHeaders.setRawHeaders(
+            networkString('if-modified-since'),
+                          [b'01 Jan 1999 00:00:01 GMT']
+            )
+        req.setLastModified(1)
+
+        result = req.setLastModified(0)
+
+        self.assertEqual(result, http.CACHED)
+
+
     def test_setHost(self):
         """
         L{http.Request.setHost} sets the value of the host request header.
